@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 
-/* $Id: client.c,v 1.17 2005/07/27 12:01:46 evertonm Exp $ */
+/* $Id: client.c,v 1.24 2005/08/12 23:30:58 evertonm Exp $ */
 
 #include <stdio.h>
 #include <sys/types.h>
@@ -32,9 +32,11 @@ Boston, MA 02111-1307, USA.
 #include "sock.h"
 #include "pipe.h"
 #include "common.h"
+#include "usock.h"
 
-extern nepim_pipe_set_t pipes; /* from server.c */
-extern nepim_slot_set_t slots; /* from server.c */
+extern nepim_pipe_set_t pipes;    /* from server.c */
+extern nepim_slot_set_t slots;    /* from server.c */
+extern nepim_usock_set_t udp_tab; /* from server.c */
 
 static void *on_tcp_interval(oop_source *src, struct timeval tv, void *user);
 static void tcp_pipe_stop(int sd);
@@ -70,6 +72,8 @@ static void *on_tcp_read(oop_source *src, int sd,
   assert(event == OOP_READ);
   assert(sd == pipe->sd);
 
+  assert(sizeof(buf) == nepim_global.tcp_read_size);
+
   rd = read(sd, buf, nepim_global.tcp_read_size);
   if (rd < 1) {
     switch (errno) {
@@ -91,6 +95,9 @@ static void *on_tcp_read(oop_source *src, int sd,
     return OOP_CONTINUE;
   }
 
+  assert(rd > 0);
+  assert(rd <= sizeof(buf));
+
   session = &pipe->session;
 
   session->byte_total_recv += rd;
@@ -113,6 +120,8 @@ static void *on_tcp_write(oop_source *src, int sd,
   assert(sd == pipe->sd);
   assert(session->max_bit_rate < 1);
 
+  assert(sizeof(buf) == nepim_global.tcp_write_size);
+
   wr = write(sd, buf, nepim_global.tcp_write_size);
   if (wr < 1) {
     switch (errno) {
@@ -137,6 +146,9 @@ static void *on_tcp_write(oop_source *src, int sd,
     return OOP_CONTINUE;
   }
 
+  assert(wr > 0);
+  assert(wr <= sizeof(buf));
+
   session->byte_total_sent += wr;
   session->byte_interval_sent += wr;
   ++session->total_writes;
@@ -158,7 +170,13 @@ static void *on_tcp_rate_write(oop_source *src, int sd,
   assert(sd == pipe->sd);
   assert(session->max_bit_rate > 0);
 
+  assert(sizeof(buf) == nepim_global.tcp_write_size);
+  
+  assert(session->rate_remaining > 0);
+
   to_write = NEPIM_MIN(session->rate_remaining, nepim_global.tcp_write_size);
+  assert(to_write > 0);
+  assert(to_write <= sizeof(buf));
 
   wr = write(sd, buf, to_write);
   if (wr < 1) {
@@ -184,6 +202,10 @@ static void *on_tcp_rate_write(oop_source *src, int sd,
     return OOP_CONTINUE;
   }
 
+  assert(wr > 0);
+  assert(wr <= sizeof(buf));
+  assert(wr <= to_write);
+
   session->byte_total_sent += wr;
   session->byte_interval_sent += wr;
   session->rate_remaining -= wr;
@@ -205,7 +227,7 @@ static void *on_tcp_rate_write(oop_source *src, int sd,
 
     /* schedule for next time saved by on_tcp_rate_delay() */
     nepim_global.oop_src->on_time(nepim_global.oop_src,
-				  session->tv_rate,
+				  session->tv_send_rate,
 				  on_tcp_rate_delay, pipe);
   }
 
@@ -216,27 +238,20 @@ static void *on_tcp_rate_delay(oop_source *src, struct timeval tv, void *user)
 {
   nepim_pipe_t *pipe = user;
   nepim_session_t *session = &pipe->session;
-  long long tmp;
 
-  assert(timercmp(&tv, &session->tv_rate, ==));
+  assert(timercmp(&tv, &session->tv_send_rate, ==));
   assert(session->max_bit_rate > 0);
 
   /* save next scheduling time */
   {
-    int result = gettimeofday(&session->tv_rate, 0);
+    int result = gettimeofday(&session->tv_send_rate, 0);
     assert(!result);
   }
-  session->tv_rate.tv_usec += session->write_delay;
-  if (session->tv_rate.tv_usec >= 1000000) {
-    session->tv_rate.tv_usec -= 1000000;
-    ++session->tv_rate.tv_sec;
-  }
+  nepim_timer_usec_add(&session->tv_send_rate, session->write_delay);
 
   /* calculate bytes to be written from rate */
-  tmp = session->max_bit_rate;
-  tmp *= session->write_delay;
-  tmp /= 8000000;
-  session->rate_remaining = tmp;
+  session->rate_remaining = nepim_bps2bytes(session->max_bit_rate,
+					    session->write_delay);
 
 #if 0
   fprintf(stderr, 
@@ -314,20 +329,20 @@ static void *on_tcp_interval(oop_source *src, struct timeval tv, void *user)
 static int greet(int sd)
 {
   char buf[1024];
+  nepim_greet_t opt;
   int pr;
   int wr;
 
-  pr = snprintf(buf, sizeof(buf),
-		"hello server_send=%d bit_rate=%lld "
-		"stat_interval=%d test_duration=%d "
-		"write_delay=%ld\n", 
-		nepim_global.duplex_mode || !nepim_global.simplex_client_send,
-		nepim_global.bit_rate,
-		nepim_global.stat_interval,
-		nepim_global.test_duration,
-		nepim_global.write_delay);
+  opt.must_send     = nepim_global.duplex_mode || !nepim_global.simplex_client_send;
+  opt.bit_rate      = nepim_global.bit_rate;
+  opt.stat_interval = nepim_global.stat_interval;
+  opt.test_duration = nepim_global.test_duration;
+  opt.write_delay   = nepim_global.write_delay;
+
+  pr = nepim_write_greetings(&opt, buf, sizeof(buf));
+  if (pr < 0)
+    return -1;
   assert(pr > 0);
-  assert(pr < sizeof(buf));
 
   fprintf(stderr, "%d: sending: %s", sd, buf);
 
@@ -352,9 +367,9 @@ static void tcp_pipe_start(int sd)
 
   if (session->must_send) {
     if (session->max_bit_rate > 0) {
-      session->tv_rate = OOP_TIME_NOW;
+      session->tv_send_rate = OOP_TIME_NOW;
       nepim_global.oop_src->on_time(nepim_global.oop_src,
-				    session->tv_rate,
+				    session->tv_send_rate,
 				    on_tcp_rate_delay, pipe);
     }
     else {
@@ -398,7 +413,7 @@ static void tcp_pipe_stop(int sd)
 
       /* stop periodic write scheduler */
       nepim_global.oop_src->cancel_time(nepim_global.oop_src,
-					session->tv_rate,
+					session->tv_send_rate,
 					on_tcp_rate_delay, pipe);
     }
     else {
@@ -480,11 +495,13 @@ static void spawn_one_tcp_client()
     {
       nepim_greet_t opt;
 
-      opt.must_send     = nepim_global.duplex_mode || nepim_global.simplex_client_send;
-      opt.bit_rate      = nepim_global.bit_rate;
-      opt.stat_interval = nepim_global.stat_interval;
-      opt.test_duration = nepim_global.test_duration;
-      opt.write_delay   = nepim_global.write_delay;
+      opt.must_send             = nepim_global.duplex_mode || nepim_global.simplex_client_send;
+      opt.bit_rate              = nepim_global.bit_rate;
+      opt.stat_interval         = nepim_global.stat_interval;
+      opt.test_duration         = nepim_global.test_duration;
+      opt.write_delay           = nepim_global.write_delay;
+      opt.udp_keepalive_send    = nepim_global.udp_keepalive_send;
+      opt.udp_keepalive_require = nepim_global.udp_keepalive_require;
 
       nepim_pipe_set_add(&pipes, sd, &opt);
     }
@@ -532,10 +549,12 @@ void nepim_client_run()
 {
   void *result;
 
-  nepim_pipe_set_init(&pipes); /* TCP/UDP sockets storage */
+  nepim_pipe_set_init(&pipes);
 
   if (nepim_global.udp_mode) {
-    nepim_slot_set_init(&slots); /* UDP pseudo-streams */
+    nepim_slot_set_init(&slots);
+    nepim_usock_set_init(&udp_tab);
+
     nepim_udp_clients();
   }
   else
