@@ -19,11 +19,13 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 
-/* $Id: common.c,v 1.16 2005/07/27 12:01:46 evertonm Exp $ */
+/* $Id: common.c,v 1.27 2005/08/12 23:30:58 evertonm Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <arpa/inet.h>
+#include <netinet/in.h>
 #include <assert.h>
 
 #include "common.h"
@@ -38,7 +40,7 @@ const char * const NEPIM_LABEL_TOTAL_BROKEN = "avg(broken)";
 
 const char *nepim_version()
 {
-  return "0.9";
+  return "0.10";
 }
 
 static void nepim_dump_stat(FILE *out, const char *label,
@@ -151,3 +153,29 @@ void report_broken_slot_stat(FILE *out, const nepim_slot_t *slot)
 		  session->total_reads,
 		  session->total_writes);
 }
+
+#define NEPIM_MEGA (1000000)
+
+void nepim_timer_usec_add(struct timeval *tv, suseconds_t usec)
+{
+  tv->tv_usec += usec;
+
+  /* overflow? */
+  if (tv->tv_usec >= NEPIM_MEGA) {
+    int sec = tv->tv_usec / NEPIM_MEGA;
+    tv->tv_usec %= NEPIM_MEGA;
+    tv->tv_sec += sec;
+  }
+
+  assert(tv->tv_usec < NEPIM_MEGA);
+}
+
+long long nepim_bps2bytes(long long bps_bit_rate, suseconds_t usec_delay)
+{
+  long long bytes;
+
+  bytes = bps_bit_rate * usec_delay;
+  bytes /= 8000000;
+
+  return bytes;
+}
