@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 
-/* $Id: common.h,v 1.8 2005/07/27 12:01:46 evertonm Exp $ */
+/* $Id: common.h,v 1.13 2005/08/10 11:18:27 evertonm Exp $ */
 
 #ifndef NEPIM_COMMON_H
 #define NEPIM_COMMON_H
@@ -48,5 +48,8 @@ void nepim_slot_stat(FILE *out, const char *label, int sd,
 void report_broken_pipe_stat(FILE *out, const nepim_pipe_t *pipe);
 void report_broken_slot_stat(FILE *out, const nepim_slot_t *slot);
 
+void nepim_timer_usec_add(struct timeval *tv, suseconds_t usec);
+
+long long nepim_bps2bytes(long long bps_bit_rate, suseconds_t usec_delay);
 
 #endif /* NEPIM_COMMON_H */
