@@ -45,6 +45,12 @@ nepim_global_t nepim_global = {
   0,       /* join_list */
   -1,      /* ttl */
   -1,      /* mcast_ttl */
+  500000,  /* udp_greet_delay (usec) */
+  3,       /* max_greetings */
+  2000000, /* udp_keepalive_timer (usec, receivers) */
+  500000,  /* udp_keepalive_delay (usec, non-senders) */
+  1,       /* udp_keepalive_require (boolean, receivers) */
+  1,       /* udp_keepalive_send (boolean, non-senders) */
 
   0, /* oop source */
   0  /* source interface */
