@@ -50,6 +50,12 @@ struct nepim_global_t {
   char       *join_list;
   int         ttl;
   int         mcast_ttl;
+  suseconds_t udp_greet_delay; /* usec */
+  int         max_greetings;
+  suseconds_t udp_keepalive_timer; /* usec, receivers */
+  suseconds_t udp_keepalive_delay; /* usec, non-senders */
+  int         udp_keepalive_require; /* boolean, receivers */
+  int         udp_keepalive_send; /* boolean, non-senders */
 
   oop_source_sys *oop_sys;
   oop_source     *oop_src;
