@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 
-/* $Id: greet.h,v 1.2 2005/07/27 11:36:33 evertonm Exp $ */
+/* $Id: greet.h,v 1.4 2005/08/12 23:30:58 evertonm Exp $ */
 
 #ifndef NEPIM_GREET_H
 #define NEPIM_GREET_H
@@ -32,8 +32,13 @@ struct nepim_greet_t {
   int  stat_interval;
   int  test_duration;
   long write_delay;
+  int  udp_keepalive_send;
+  int  udp_keepalive_require;
 };
 
+int nepim_write_greetings(const nepim_greet_t *opt, char *buf,
+			  int buf_size);
+
 int nepim_parse_greetings(nepim_greet_t *opt, 
 			  const char *buf, const char *past_end);
 
