@@ -21,7 +21,7 @@ Boston, MA 02111-1307, USA.
 /*
   nepim - network pipemeter
 
-  $Id: main.c,v 1.13 2005/07/27 13:31:45 evertonm Exp $
+  $Id: main.c,v 1.14 2005/08/11 10:47:13 evertonm Exp $
  */
 
 #include <stdlib.h>
@@ -42,6 +42,7 @@ typedef void (*sighandler_t)(int);
 #include "conf.h"
 #include "common.h"
 #include "str.h"
+#include "int.h"
 
 extern void nepim_server_run();
 extern void nepim_client_run();
@@ -152,6 +153,8 @@ int main(int argc, const char *argv[])
 {
   int i;
 
+  nepim_int_sanity();
+
   for (i = 1; i < argc; ++i) {
     const char *arg = argv[i];
 
