@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 
-/* $Id: pipe.c,v 1.4 2005/07/26 11:41:50 evertonm Exp $ */
+/* $Id: pipe.c,v 1.5 2005/07/29 23:08:58 evertonm Exp $ */
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -40,7 +40,7 @@ nepim_pipe_t *nepim_pipe_set_get(const nepim_pipe_set_t *set, int sd)
 void nepim_pipe_set_add(nepim_pipe_set_t *set, int sd, 
 			const nepim_greet_t *opt)
 {
-  nepim_pipe_t *pipe = malloc(sizeof(nepim_pipe_t));
+  nepim_pipe_t *pipe = malloc(sizeof(*pipe));
   assert(pipe);
 
   pipe->sd = sd;
