@@ -19,47 +19,24 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 
-/* $Id: pipe.h,v 1.3 2005/04/19 03:40:33 evertonm Exp $ */
+/* $Id: pipe.h,v 1.5 2005/07/26 11:41:50 evertonm Exp $ */
 
 #ifndef NEPIM_PIPE_H
 #define NEPIM_PIPE_H
 
-#include <sys/time.h>
+#include "array.h"
+#include "session.h"
 
 typedef struct nepim_pipe_t nepim_pipe_t;
 typedef struct nepim_pipe_set_t nepim_pipe_set_t;
 
 struct nepim_pipe_t {
-  int sd;
-  int must_send;
-  long long max_bit_rate;
-  int stat_interval;
-  int test_duration;
-  suseconds_t write_delay; /* usec, max_bit_rate > 0 */
-  int duration_done; /* boolean */
-
-  int rate_remaining; /* bytes, max_bit_rate > 0 */
-
-  int total_reads;
-  int total_writes;
-  int interval_reads;
-  int interval_writes;
-
-  long long byte_total_sent;
-  long long byte_total_recv;
-
-  long long byte_interval_sent;
-  long long byte_interval_recv;
-
-  struct timeval tv_duration;
-  struct timeval tv_interval;
-  struct timeval tv_start;
-  struct timeval tv_rate;
+  int             sd;
+  nepim_session_t session;
 };
 
 struct nepim_pipe_set_t {
-  nepim_pipe_t **head;
-  int          capacity;
+  nepim_array_t array;
 };
 
 void nepim_pipe_set_init(nepim_pipe_set_t *set);
@@ -67,11 +44,7 @@ void nepim_pipe_set_init(nepim_pipe_set_t *set);
 nepim_pipe_t *nepim_pipe_set_get(const nepim_pipe_set_t *set, int sd);
 
 void nepim_pipe_set_add(nepim_pipe_set_t *set, int sd, 
-			int must_send, 
-			long long max_bit_rate, 
-			int stat_interval,
-			int test_duration,
-			suseconds_t write_delay);
+			const nepim_greet_t *opt);
 
 void nepim_pipe_set_del(nepim_pipe_set_t *set, int sd);
 
