@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 
-/* $Id: server.c,v 1.22 2005/07/27 12:01:46 evertonm Exp $ */
+/* $Id: server.c,v 1.31 2005/08/12 23:30:58 evertonm Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -59,6 +59,8 @@ static void *on_tcp_read(oop_source *src, int sd,
   nepim_pipe_t *pipe = user;
   nepim_session_t *session = &pipe->session;
 
+  assert(nepim_global.tcp_read_size = sizeof(buf));
+
   rd = read(sd, buf, nepim_global.tcp_read_size);
   if (rd < 1) {
     switch (errno) {
@@ -80,6 +82,9 @@ static void *on_tcp_read(oop_source *src, int sd,
     return OOP_CONTINUE;
   }
 
+  assert(rd > 0);
+  assert(rd <= sizeof(buf));
+
   session->byte_total_recv += rd;
   session->byte_interval_recv += rd;
   ++session->total_reads;
@@ -96,6 +101,12 @@ static void *on_tcp_write(oop_source *src, int sd,
   nepim_pipe_t *pipe = user;
   nepim_session_t *session = &pipe->session;
 
+  assert(event == OOP_WRITE);
+  assert(sd == pipe->sd);
+  assert(session->max_bit_rate < 1);
+
+  assert(sizeof(buf) == nepim_global.tcp_write_size);
+
   wr = write(sd, buf, nepim_global.tcp_write_size);
   if (wr < 1) {
     switch (errno) {
@@ -120,6 +131,9 @@ static void *on_tcp_write(oop_source *src, int sd,
     return OOP_CONTINUE;
   }
 
+  assert(wr > 0);
+  assert(wr <= sizeof(buf));
+
   session->byte_total_sent += wr;
   session->byte_interval_sent += wr;
   ++session->total_writes;
@@ -141,7 +155,12 @@ static void *on_tcp_rate_write(oop_source *src, int sd,
   assert(sd == pipe->sd);
   assert(session->max_bit_rate > 0);
 
+  assert(sizeof(buf) == nepim_global.tcp_write_size);
+  assert(session->rate_remaining > 0);
+
   to_write = NEPIM_MIN(session->rate_remaining, nepim_global.tcp_write_size);
+  assert(to_write > 0);
+  assert(to_write <= sizeof(buf));
 
   wr = write(sd, buf, to_write);
   if (wr < 1) {
@@ -167,6 +186,10 @@ static void *on_tcp_rate_write(oop_source *src, int sd,
     return OOP_CONTINUE;
   }
 
+  assert(wr > 0);
+  assert(wr <= sizeof(buf));
+  assert(wr <= to_write);
+
   session->byte_total_sent += wr;
   session->byte_interval_sent += wr;
   session->rate_remaining -= wr;
@@ -188,7 +211,7 @@ static void *on_tcp_rate_write(oop_source *src, int sd,
 
     /* schedule for next time saved by on_tcp_rate_delay() */
     nepim_global.oop_src->on_time(nepim_global.oop_src,
-				  session->tv_rate,
+				  session->tv_send_rate,
 				  on_tcp_rate_delay, pipe);
   }
 
@@ -199,27 +222,20 @@ static void *on_tcp_rate_delay(oop_source *src, struct timeval tv, void *user)
 {
   nepim_pipe_t *pipe = user;
   nepim_session_t *session = &pipe->session;
-  long long tmp;
 
-  assert(timercmp(&tv, &session->tv_rate, ==));
+  assert(timercmp(&tv, &session->tv_send_rate, ==));
   assert(session->max_bit_rate > 0);
 
   /* save next scheduling time */
   {
-    int result = gettimeofday(&session->tv_rate, 0);
+    int result = gettimeofday(&session->tv_send_rate, 0);
     assert(!result);
   }
-  session->tv_rate.tv_usec += session->write_delay;
-  if (session->tv_rate.tv_usec >= 1000000) {
-    session->tv_rate.tv_usec -= 1000000;
-    ++session->tv_rate.tv_sec;
-  }
+  nepim_timer_usec_add(&session->tv_send_rate, session->write_delay);
 
   /* calculate bytes to be written from rate */
-  tmp = session->max_bit_rate;
-  tmp *= session->write_delay;
-  tmp /= 8000000;
-  session->rate_remaining = tmp;
+  session->rate_remaining = nepim_bps2bytes(session->max_bit_rate,
+					    session->write_delay);
 
 #if 0
   fprintf(stderr, 
@@ -301,10 +317,11 @@ static int parse_greetings(int sd, const char *buf, const char *past_end)
 
   fprintf(stderr, 
 	  "%d: TCP incoming: send=%d max_rate=%lld "
-	  "interval=%d duration=%d delay=%ld\n", 
+	  "interval=%d duration=%d delay=%ld ka_send=%d ka_req=%d\n", 
 	  sd, opt.must_send, opt.bit_rate, 
 	  opt.stat_interval, opt.test_duration, 
-	  opt.write_delay);
+	  opt.write_delay, opt.udp_keepalive_send,
+	  opt.udp_keepalive_require);
 
   nepim_pipe_set_add(&pipes, sd, &opt);
 
@@ -394,9 +411,9 @@ static void tcp_pipe_start(int sd)
 
   if (session->must_send) {
     if (session->max_bit_rate > 0) {
-      session->tv_rate = OOP_TIME_NOW;
+      session->tv_send_rate = OOP_TIME_NOW;
       nepim_global.oop_src->on_time(nepim_global.oop_src,
-				    session->tv_rate,
+				    session->tv_send_rate,
 				    on_tcp_rate_delay, pipe);
     }
     else {
@@ -457,7 +474,7 @@ static void tcp_pipe_cancel_io(int sd)
       
       /* stop periodic write scheduler */
       nepim_global.oop_src->cancel_time(nepim_global.oop_src,
-					session->tv_rate,
+					session->tv_send_rate,
 					on_tcp_rate_delay, pipe);
     }
     else {
@@ -578,8 +595,8 @@ static int spawn_tcp_listener(const char *hostname, const char *portname)
 
 #if 0
     fprintf(stderr, 
-	    "TCP listener: host=%s,%s len=%d family=%d type=%d proto=%d\n",
-	    hostname, portname, 
+	    "TCP listener: host=%s,%s(%d) len=%d family=%d type=%d proto=%d\n",
+	    hostname, portname, nepim_sock_get_port(ai->ai_addr),
 	    ai->ai_addrlen, ai->ai_family, 
 	    ai->ai_socktype, ai->ai_protocol);
 #endif
@@ -595,8 +612,9 @@ static int spawn_tcp_listener(const char *hostname, const char *portname)
 				      nepim_global.ttl);
     if (sd < 0) {
       fprintf(stderr, 
-	      "%s: TCP listener socket failed for %s,%s: %d\n",
-	      __PRETTY_FUNCTION__, hostname, portname, sd);
+	      "%s: TCP listener socket failed for %s,%s(%d): %d\n",
+	      __PRETTY_FUNCTION__, hostname, portname, 
+	      nepim_sock_get_port(ai->ai_addr), sd);
       break;
     }
 
@@ -607,8 +625,8 @@ static int spawn_tcp_listener(const char *hostname, const char *portname)
     ++tcp_listeners;
 
     fprintf(stderr, 
-	    "%d: TCP socket listening on %s,%s\n",
-	    sd, hostname, portname);
+	    "%d: TCP socket listening on %s,%s(%d)\n",
+	    sd, hostname, portname, nepim_sock_get_port(ai->ai_addr));
   }
 
   freeaddrinfo(ai_res);
@@ -650,29 +668,30 @@ void nepim_server_run()
   int tcp_listeners = 0;
   int udp_listeners = 0;
 
+  /* these must be initialized before spawning listeners */
+  nepim_pipe_set_init(&pipes);
+  nepim_slot_set_init(&slots);
+  nepim_usock_set_init(&udp_tab);
+
   if (nepim_global.bind_list)
-    parse_listeners(nepim_global.bind_list, &udp_listeners, &tcp_listeners, 0 /* join */);
+    parse_listeners(nepim_global.bind_list, &udp_listeners, &tcp_listeners,
+		    0 /* no join */);
 
   if (nepim_global.join_list)
-    parse_listeners(nepim_global.join_list, &udp_listeners, &tcp_listeners, 1 /* join */);
+    parse_listeners(nepim_global.join_list, &udp_listeners, &tcp_listeners,
+		    1 /* do join */);
 
   if (!nepim_global.bind_list && !nepim_global.join_list) {
     if (!nepim_global.no_inet6)
       tcp_listeners += spawn_tcp_listener(INET6_ANY, nepim_global.portname);
     tcp_listeners += spawn_tcp_listener(INET_ANY, nepim_global.portname);
     if (!nepim_global.no_inet6)
-      udp_listeners += nepim_udp_listener(INET6_ANY, nepim_global.portname, 0 /* no join */);
-    udp_listeners += nepim_udp_listener(INET_ANY, nepim_global.portname, 0 /* no join */);
+      udp_listeners += nepim_udp_listener(INET6_ANY, nepim_global.portname,
+					  0 /* no join */);
+    udp_listeners += nepim_udp_listener(INET_ANY, nepim_global.portname,
+					0 /* no join */);
   }
-
-  if (tcp_listeners)
-    nepim_pipe_set_init(&pipes);
-
-  if (udp_listeners) {
-    nepim_slot_set_init(&slots);
-    nepim_usock_set_init(&udp_tab);
-  }
-
+  
   if (tcp_listeners || udp_listeners)
     fprintf(stderr, "%s: server ready\n", nepim_global.prog_name);
 
