@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 
-/* $Id: session.c,v 1.2 2005/07/27 11:36:33 evertonm Exp $ */
+/* $Id: session.c,v 1.3 2005/08/12 23:30:58 evertonm Exp $ */
 
 #include <assert.h>
 
@@ -41,11 +41,13 @@ void nepim_session_init(nepim_session_t *session, const nepim_greet_t *opt,
 
   session->type               = type;
 
-  session->must_send          = opt->must_send;
-  session->max_bit_rate       = opt->bit_rate;
-  session->stat_interval      = opt->stat_interval;
-  session->test_duration      = opt->test_duration;
-  session->write_delay        = opt->write_delay;
+  session->must_send             = opt->must_send;
+  session->max_bit_rate          = opt->bit_rate;
+  session->stat_interval         = opt->stat_interval;
+  session->test_duration         = opt->test_duration;
+  session->write_delay           = opt->write_delay;
+  session->udp_keepalive_send    = opt->udp_keepalive_send;
+  session->udp_keepalive_require = opt->udp_keepalive_require;
 
   session->duration_done      = 0;
   session->total_reads        = 0;
