@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 
-/* $Id: session.h,v 1.1 2005/07/24 22:14:32 evertonm Exp $ */
+/* $Id: session.h,v 1.5 2005/08/13 00:18:29 evertonm Exp $ */
 
 #ifndef NEPIM_SESSION_H
 #define NEPIM_SESSION_H
@@ -46,6 +46,8 @@ struct nepim_session_t {
   int test_duration;
   suseconds_t write_delay; /* usec, used when max_bit_rate > 0 */
   int duration_done;       /* boolean */
+  int udp_keepalive_send;  /* boolean, non-senders */
+  int udp_keepalive_require; /* boolean, receivers */
 
   int rate_remaining;      /* bytes, used when max_bit_rate > 0 */
 
@@ -63,7 +65,10 @@ struct nepim_session_t {
   struct timeval tv_duration;
   struct timeval tv_interval;
   struct timeval tv_start;
-  struct timeval tv_rate;
+  struct timeval tv_send_rate;
+  struct timeval tv_greet_rate; /* udp client */
+  struct timeval tv_keepalive; /* udp non-sender keepalive period */
+  struct timeval tv_keepalive_timer; /* udp keepalive receiver */
 };
 
 void nepim_session_init(nepim_session_t *session, const nepim_greet_t *opt,
