@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 
-/* $Id: sock.c,v 1.16 2005/07/27 20:38:33 evertonm Exp $ */
+/* $Id: sock.c,v 1.18 2005/08/05 11:12:13 evertonm Exp $ */
 
 
 #include <assert.h>
@@ -33,6 +33,7 @@ Boston, MA 02111-1307, USA.
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <netinet/tcp.h>
+#include <arpa/inet.h>
 
 #include "sock.h"
 
@@ -72,7 +73,7 @@ Boston, MA 02111-1307, USA.
 #define NEPIM_SOCK_ERR_MCAST_TTL      (-13)
 #define NEPIM_SOCK_ERR_MCAST_JOIN     (-14)
 
-static int get_port(struct sockaddr *addr)
+int nepim_sock_get_port(const struct sockaddr *addr)
 {
   union {
     struct sockaddr_in inet;
@@ -244,8 +245,11 @@ int nepim_create_socket(struct sockaddr *addr,
   }
 
 #if 0
-  fprintf(stderr, "DEBUG %s %s bind(port=%d)\n",
-	  __FILE__, __PRETTY_FUNCTION__, get_port(addr));
+  fprintf(stderr, "DEBUG %s %s bind: sd=%d af=%d proto=%d addr=%s port=%d\n",
+	  __FILE__, __PRETTY_FUNCTION__, sd, 
+	  family, protocol,
+	  inet_ntoa(((struct sockaddr_in *) addr)->sin_addr),
+	  ntohs(((struct sockaddr_in *) addr)->sin_port));
 #endif
 
   if (bind(sd, addr, addr_len)) {
@@ -253,6 +257,14 @@ int nepim_create_socket(struct sockaddr *addr,
     return NEPIM_SOCK_ERR_BIND;
   }
 
+#if 0
+  fprintf(stderr, "DEBUG %s %s bound: sd=%d af=%d proto=%d addr=%s port=%d\n",
+	  __FILE__, __PRETTY_FUNCTION__, sd, 
+	  family, protocol,
+	  inet_ntoa(((struct sockaddr_in *) addr)->sin_addr),
+	  ntohs(((struct sockaddr_in *) addr)->sin_port));
+#endif
+
   return sd;
 }
 
@@ -345,6 +357,17 @@ int nepim_connect_client_socket(struct sockaddr *addr,
   if (sd < 0)
     return sd;
 
+#ifdef SO_BSDCOMPAT
+  /*
+   * We don't want Linux ECONNREFUSED on UDP sockets
+   */
+  if (protocol == IPPROTO_UDP) {
+    int one = 1;
+    if (setsockopt(sd, SOL_SOCKET, SO_BSDCOMPAT, &one, sizeof(one)))
+      return -1;
+ }
+#endif /* Linux SO_BSDCOMPAT */
+
   result = nepim_socket_block(sd);
   if (result) {
     close(sd);
@@ -353,7 +376,7 @@ int nepim_connect_client_socket(struct sockaddr *addr,
 
   fprintf(stderr, 
 	  "DEBUG FIXME %s %s slow synchronous connect(port=%d)\n",
-	  __FILE__, __PRETTY_FUNCTION__, get_port(addr));
+	  __FILE__, __PRETTY_FUNCTION__, nepim_sock_get_port(addr));
 
   if (connect(sd, addr, addr_len)) {
     close(sd);
