@@ -19,13 +19,15 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 
-/* $Id: udp_server.c,v 1.21 2005/07/27 18:51:12 evertonm Exp $ */
+/* $Id: udp_server.c,v 1.51 2005/08/13 02:17:16 evertonm Exp $ */
 
 
 #include <stdio.h>
 #include <string.h>
 #include <sys/types.h>
 #include <sys/socket.h>
+#include <netinet/in.h>
+#include <arpa/inet.h>
 #include <netdb.h>
 #include <errno.h>
 #include <oop.h>
@@ -36,17 +38,26 @@ Boston, MA 02111-1307, USA.
 #include "common.h"
 #include "slot.h"
 #include "usock.h"
+#include "udp_header.h"
 
-
-/* version(1) + local_slot(2) + remote_lost(2) + seq(4) */
-const int UDP_HEADER_LEN = 9; 
-
-extern nepim_slot_set_t slots; /* from server.c */
+extern nepim_slot_set_t  slots;   /* from server.c */
 extern nepim_usock_set_t udp_tab; /* from server.c */
 
 static void schedule_stat_interval(int local_slot);
-static void udp_slot_cancel_timers(nepim_slot_t *slot);
-static void *on_udp_rate_delay(oop_source *src, struct timeval tv, void *user);
+static void udp_slot_cancel_stat_timers(nepim_slot_t *slot);
+static void *on_udp_rate_delay(oop_source *src, struct timeval tv,
+			       void *user);
+static void *on_udp_write(oop_source *src, int sd,
+			  oop_event event, void *user);
+static void cancel_slot_rate_write(nepim_slot_t *slot);
+static void cancel_slot_write(nepim_slot_t *slot);
+static void udp_slot_cancel_writers(nepim_slot_t *slot);
+static void *on_udp_keepalive_time(oop_source *src, struct timeval tv,
+				   void *user);
+static void *on_udp_keepalive_require(oop_source *src, struct timeval tv,
+				      void *user);
+static void udp_slot_kill(nepim_slot_t *slot);
+
 
 static void *on_udp_interval(oop_source *src, struct timeval tv, void *user)
 {
@@ -70,7 +81,7 @@ static void *on_udp_interval(oop_source *src, struct timeval tv, void *user)
   session->interval_reads     = 0;
   session->interval_writes    = 0;
 
-  schedule_stat_interval(slot->udp_sd);
+  schedule_stat_interval(slot->index);
 
   return OOP_CONTINUE;
 }
@@ -119,12 +130,23 @@ static void *on_udp_duration(oop_source *src, struct timeval tv, void *user)
   session->total_writes    = 0;
   session->duration_done   = 1;
 
-  udp_slot_cancel_timers(slot);
+#if 0
+  udp_slot_cancel_stat_timers(slot);
+  udp_slot_cancel_writers(slot);
+
+  fprintf(stderr, 
+	  "DEBUG FIXME %s %s: (%d-%d) finished, "
+	  "not killed (beware possibly active remote sender)\n",
+	  __FILE__, __PRETTY_FUNCTION__,
+	  slot->index, slot->index_remote);
+#else
+  udp_slot_kill(slot);
+#endif
 
   return OOP_CONTINUE;
 }
 
-static void udp_slot_cancel_timers(nepim_slot_t *slot)
+static void udp_slot_cancel_stat_timers(nepim_slot_t *slot)
 {
   nepim_session_t *session = &slot->session;
 
@@ -137,86 +159,103 @@ static void udp_slot_cancel_timers(nepim_slot_t *slot)
 				    on_udp_interval, slot);
 }
 
-static void udp_slot_cancel_io(nepim_slot_t *slot)
+static void udp_slot_cancel_writers(nepim_slot_t *slot)
 {
   nepim_session_t *session = &slot->session;
-  int sd = slot->udp_sd;
-
-  nepim_global.oop_src->cancel_fd(nepim_global.oop_src,
-				  sd, OOP_READ);
 
   if (session->must_send) {
     if (session->max_bit_rate > 0) {
       /* stop current writing, if any */
-      nepim_global.oop_src->cancel_fd(nepim_global.oop_src,
-				      sd, OOP_WRITE);
+      if (slot->want_write) {
+	assert(slot->want_write == 1);
+	cancel_slot_rate_write(slot); /* rate write */
+      }
       
       /* stop periodic write scheduler */
       nepim_global.oop_src->cancel_time(nepim_global.oop_src,
-					session->tv_rate,
+					session->tv_send_rate,
 					on_udp_rate_delay, slot);
     }
     else {
-      nepim_global.oop_src->cancel_fd(nepim_global.oop_src,
-				      sd, OOP_WRITE);
+      cancel_slot_write(slot); /* full write */
     }
   }
+  else {
+    if (session->udp_keepalive_send) {
+      if (slot->want_write) {
+	assert(slot->want_write == 1);
+	nepim_cancel_slot_keepalive(slot); /* keepalive write */
+      }
+
+      /* stop periodic keepalive scheduler */
+      nepim_global.oop_src->cancel_time(nepim_global.oop_src,
+					session->tv_keepalive,
+					on_udp_keepalive_time, slot);
+    }
+  }
+
+  assert(!slot->want_write);
+}
+
+static void udp_slot_cancel_io(nepim_slot_t *slot)
+{
+  udp_slot_cancel_writers(slot);
 }
 
 static void udp_slot_kill(nepim_slot_t *slot)
 {
   int slot_index = slot->index;
 
-  udp_slot_cancel_timers(slot);
+  udp_slot_cancel_stat_timers(slot);
   udp_slot_cancel_io(slot);
 
+  if (slot->session.udp_keepalive_require)
+    nepim_global.oop_src->cancel_time(nepim_global.oop_src,
+				      slot->session.tv_keepalive_timer,
+				      on_udp_keepalive_require, slot);
+
   nepim_sock_show_opt(stderr, slot->udp_sd);
 
   nepim_slot_set_del(&slots, slot_index);
 }
 
-static int udp_write(nepim_slot_t *slot, size_t len)
-{
-  char buf[len];
-  ssize_t wr;
-
-  assert(sizeof(buf) == len);
-  assert(len >= UDP_HEADER_LEN);
-
-  *buf                   = 0; /* version=0 */
-  *(uint16_t *)(buf + 1) = htons(slot->index_remote);
-  *(uint16_t *)(buf + 3) = htons(slot->index);
-  *(uint32_t *)(buf + 5) = htonl(slot->seq++);
-
-  wr = sendto(slot->udp_sd, buf, len, 0,
-	      (const struct sockaddr *) &slot->remote,
-	      slot->remote_len);
-
-  return wr;
-}
-
+#if 0
 static void *on_udp_rate_write(oop_source *src, int sd,
 			       oop_event event, void *user)
 {
+  char buf[nepim_global.udp_write_size];
   int wr;
-  nepim_slot_t *slot = user;
-  nepim_session_t *session = &slot->session;
+  nepim_slot_t *slot;
+  nepim_session_t *session;
   int to_write;
 
+  assert(nepim_global.udp_write_size == sizeof(buf));
+  assert(!user);
   assert(event == OOP_WRITE);
-  assert(sd == slot->udp_sd);
+
+  slot = nepim_slot_find_next_writer(&slots, sd);
+  assert(slot);
+
+  session = &slot->session;
+
   assert(session->max_bit_rate > 0);
 
+  assert(sd == slot->udp_sd);
+  assert(session->rate_remaining > 0);
+  
   to_write = NEPIM_MIN(session->rate_remaining, nepim_global.udp_write_size);
+  assert(to_write > 0);
+  assert(to_write <= session->rate_remaining);
+  assert(to_write <= nepim_global.udp_write_size);
 
-  wr = udp_write(slot, to_write);
+  wr = nepim_slot_buf_sendto(slot, buf, to_write, UDP_TYPE_DATA);
   if (wr < 1) {
     switch (errno) {
     case EINTR:
-      fprintf(stderr, "rate_write: EINTR on UDP socket %d\n", sd);
-      return OOP_CONTINUE;
     case EAGAIN:
-      fprintf(stderr, "rate_write: EAGAIN on UDP socket %d\n", sd);
+      nepim_usock_write_error(&udp_tab, sd, 
+			      slot->index, slot->index_remote,
+			      errno);
       return OOP_CONTINUE;
     case EPIPE:
       fprintf(stderr, "rate_write: EPIPE on UDP socket %d\n", sd);
@@ -233,59 +272,118 @@ static void *on_udp_rate_write(oop_source *src, int sd,
     return OOP_CONTINUE;
   }
 
+  assert(wr > 0);
+  assert(wr <= to_write);
+
+  nepim_usock_write_good(&udp_tab, sd);
+
   session->byte_total_sent += wr;
   session->byte_interval_sent += wr;
   session->rate_remaining -= wr;
   ++session->total_writes;
   ++session->interval_writes;
 
-#if 0
-  fprintf(stderr, 
-	  "DEBUG %s %s %d: to_write=%d written=%d missing=%d\n",
-	  __FILE__, __PRETTY_FUNCTION__,
-	  sd, to_write, wr, session->rate_remaining);
-#endif
-
   /* finished ? */
   if (session->rate_remaining < 1) {
     /* stop writing */
-    nepim_global.oop_src->cancel_fd(nepim_global.oop_src,
-				    sd, OOP_WRITE);
+    cancel_slot_rate_write(slot);
 
     /* schedule for next time saved by on_udp_rate_delay() */
     nepim_global.oop_src->on_time(nepim_global.oop_src,
-				  session->tv_rate,
+				  session->tv_send_rate,
 				  on_udp_rate_delay, slot);
   }
 
   return OOP_CONTINUE;
 }
+#endif
+
+static void will_slot_rate_write(nepim_slot_t *slot)
+{
+  nepim_session_t *session = &slot->session;
+  int sd = slot->udp_sd;
+
+  assert(session->max_bit_rate > 0);
+
+  assert(!slot->want_write);
+  ++slot->want_write;
+
+  if (nepim_usock_writer_add(&udp_tab, sd))
+    return;
+
+  nepim_global.oop_src->on_fd(nepim_global.oop_src, sd,
+			      OOP_WRITE, on_udp_write, 0);
+}
+
+static void cancel_slot_rate_write(nepim_slot_t *slot)
+{
+  nepim_session_t *session = &slot->session;
+  int sd = slot->udp_sd;
+
+  assert(session->max_bit_rate > 0);
+
+  assert(slot->want_write == 1);
+  --slot->want_write;
+
+  if (nepim_usock_writer_del(&udp_tab, sd))
+    return;
+
+  nepim_global.oop_src->cancel_fd(nepim_global.oop_src,
+				  sd, OOP_WRITE);
+}
+
+static void will_slot_write(nepim_slot_t *slot)
+{
+  nepim_session_t *session = &slot->session;
+  int sd = slot->udp_sd;
+
+  assert(session->max_bit_rate < 1);
+
+  assert(!slot->want_write);
+  ++slot->want_write;
+
+  if (nepim_usock_writer_add(&udp_tab, sd))
+    return;
+
+  nepim_global.oop_src->on_fd(nepim_global.oop_src, sd,
+			      OOP_WRITE, on_udp_write, 0);
+}
+
+static void cancel_slot_write(nepim_slot_t *slot)
+{
+  nepim_session_t *session = &slot->session;
+  int sd = slot->udp_sd;
+
+  assert(session->max_bit_rate < 1);
+
+  assert(slot->want_write == 1);
+  --slot->want_write;
+
+  if (nepim_usock_writer_del(&udp_tab, sd))
+    return;
+
+  nepim_global.oop_src->cancel_fd(nepim_global.oop_src,
+				  sd, OOP_WRITE);
+}
 
 static void *on_udp_rate_delay(oop_source *src, struct timeval tv, void *user)
 {
   nepim_slot_t *slot = user;
   nepim_session_t *session = &slot->session;
-  long long tmp;
 
-  assert(timercmp(&tv, &session->tv_rate, ==));
+  assert(timercmp(&tv, &session->tv_send_rate, ==));
   assert(session->max_bit_rate > 0);
 
   /* save next scheduling time */
   {
-    int result = gettimeofday(&session->tv_rate, 0);
+    int result = gettimeofday(&session->tv_send_rate, 0);
     assert(!result);
   }
-  session->tv_rate.tv_usec += session->write_delay;
-  if (session->tv_rate.tv_usec >= 1000000) {
-    session->tv_rate.tv_usec -= 1000000;
-    ++session->tv_rate.tv_sec;
-  }
+  nepim_timer_usec_add(&session->tv_send_rate, session->write_delay);
 
   /* calculate bytes to be written from rate */
-  tmp = session->max_bit_rate;
-  tmp *= session->write_delay;
-  tmp /= 8000000;
-  session->rate_remaining = tmp;
+  session->rate_remaining = nepim_bps2bytes(session->max_bit_rate,
+					    session->write_delay);
 
 #if 0
   fprintf(stderr, 
@@ -300,29 +398,106 @@ static void *on_udp_rate_delay(oop_source *src, struct timeval tv, void *user)
 #endif
 
   /* start to write */
-  nepim_global.oop_src->on_fd(nepim_global.oop_src,
-			      slot->udp_sd, OOP_WRITE,
-			      on_udp_rate_write, slot);
+  will_slot_rate_write(slot);
 
   return OOP_CONTINUE;
 }
 
-static void *on_udp_write(oop_source *src, int sd,
-			  oop_event event, void *user)
+static void udp_write_rate(nepim_slot_t *slot)
 {
+  char buf[nepim_global.udp_write_size];
   int wr;
-  nepim_slot_t *slot = user;
-  nepim_session_t *session = &slot->session;
+  nepim_session_t *session;
+  int to_write;
+  int sd;
 
-  wr = udp_write(slot, nepim_global.udp_write_size);
+  assert(nepim_global.udp_write_size == sizeof(buf));
+  assert(slot);
+
+  sd = slot->udp_sd;
+  session = &slot->session;
+
+  assert(session->max_bit_rate > 0);
+  assert(session->rate_remaining > 0);
+  
+  to_write = NEPIM_MIN(session->rate_remaining, nepim_global.udp_write_size);
+  assert(to_write > 0);
+  assert(to_write <= session->rate_remaining);
+  assert(to_write <= nepim_global.udp_write_size);
+
+  wr = nepim_slot_buf_sendto(slot, buf, to_write, UDP_TYPE_DATA);
   if (wr < 1) {
     switch (errno) {
     case EINTR:
-      fprintf(stderr, "write: EINTR on UDP socket %d\n", sd);
-      return OOP_CONTINUE;
     case EAGAIN:
-      fprintf(stderr, "write: EAGAIN on UDP socket %d\n", sd);
-      return OOP_CONTINUE;
+      nepim_usock_write_error(&udp_tab, sd, 
+			      slot->index, slot->index_remote,
+			      errno);
+
+      return;
+    case EPIPE:
+      fprintf(stderr, "rate_write: EPIPE on UDP socket %d\n", sd);
+      break;
+    }
+
+    fprintf(stderr, "rate_write: connection lost on UDP socket %d\n", sd);
+
+    if (!session->duration_done)
+      report_broken_slot_stat(stdout, slot);
+
+    udp_slot_kill(slot);
+
+    return;
+  }
+
+  assert(wr > 0);
+  assert(wr <= to_write);
+
+  nepim_usock_write_good(&udp_tab, sd);
+
+  session->byte_total_sent += wr;
+  session->byte_interval_sent += wr;
+  session->rate_remaining -= wr;
+  ++session->total_writes;
+  ++session->interval_writes;
+
+  /* finished ? */
+  if (session->rate_remaining < 1) {
+    /* stop writing */
+    cancel_slot_rate_write(slot);
+
+    /* schedule for next time saved by on_udp_rate_delay() */
+    nepim_global.oop_src->on_time(nepim_global.oop_src,
+				  session->tv_send_rate,
+				  on_udp_rate_delay, slot);
+  }
+}
+
+static void udp_write_full(nepim_slot_t *slot)
+{
+  char buf[nepim_global.udp_write_size];
+  int wr;
+  nepim_session_t *session;
+  int sd;
+
+  assert(nepim_global.udp_write_size == sizeof(buf));
+  assert(slot);
+
+  sd = slot->udp_sd;
+  session = &slot->session;
+
+  assert(session->max_bit_rate < 1);
+
+  wr = nepim_slot_buf_sendto(slot, buf, sizeof(buf), UDP_TYPE_DATA);
+  if (wr < 1) {
+    switch (errno) {
+    case EINTR:
+    case EAGAIN:
+      nepim_usock_write_error(&udp_tab, sd, 
+			      slot->index, slot->index_remote,
+			      errno);
+
+      return;
     case EPIPE:
       fprintf(stderr, "write: EPIPE on UDP socket %d\n", sd);
       break;
@@ -335,13 +510,94 @@ static void *on_udp_write(oop_source *src, int sd,
 
     udp_slot_kill(slot);
 
-    return OOP_CONTINUE;
+    return;
   }
 
+  nepim_usock_write_good(&udp_tab, sd);
+
   session->byte_total_sent += wr;
   session->byte_interval_sent += wr;
   ++session->total_writes;
   ++session->interval_writes;
+}
+
+static void *on_udp_write(oop_source *src, int sd,
+			  oop_event event, void *user)
+{
+  nepim_slot_t *slot;
+  nepim_session_t *session;
+
+  assert(!user);
+  assert(event == OOP_WRITE);
+
+  slot = nepim_slot_find_next_writer(&slots, sd);
+  assert(slot);
+
+  session = &slot->session;
+
+  if (!session->must_send) {
+    if (session->udp_keepalive_send) {
+      nepim_udp_write_keepalive(slot, nepim_slot_buf_sendto,
+				on_udp_keepalive_time, udp_slot_kill);
+      return OOP_CONTINUE;
+    }
+  }
+
+  if (session->max_bit_rate > 0) {
+    udp_write_rate(slot);
+    return OOP_CONTINUE;
+  }
+
+  udp_write_full(slot);
+
+  return OOP_CONTINUE;
+}
+
+static void *on_udp_keepalive_time(oop_source *src, struct timeval tv, void *user)
+{
+  nepim_slot_t *slot = user;
+  nepim_session_t *session = &slot->session;
+
+  assert(timercmp(&tv, &session->tv_keepalive, ==));
+  assert(!session->must_send);
+  assert(session->udp_keepalive_send);
+
+  /* save next scheduling time */
+  {
+    int result = gettimeofday(&session->tv_keepalive, 0);
+    assert(!result);
+  }
+  nepim_timer_usec_add(&session->tv_keepalive, nepim_global.udp_keepalive_delay);
+
+  /* start to write */
+  nepim_will_slot_keepalive(slot, on_udp_write);
+
+  return OOP_CONTINUE;
+}
+
+static void *on_udp_keepalive_require(oop_source *src, struct timeval tv, void *user)
+{
+  nepim_slot_t *slot = user;
+  nepim_session_t *session = &slot->session;
+
+  assert(session->udp_keepalive_require);
+  assert(slot->keepalives_recv >= 0);
+
+  if (slot->keepalives_recv < 1) {
+    fprintf(stderr, 
+	    "%d %d-%d: broken slot: peer not sending keepalives?\n",
+	    slot->udp_sd, slot->index, slot->index_remote);
+
+    report_broken_slot_stat(stdout, slot);
+
+    udp_slot_kill(slot);
+
+    return OOP_CONTINUE;
+  }
+
+  slot->keepalives_recv = 0;
+
+  nepim_schedule_keepalive_timer(slot, on_udp_keepalive_require);
 
   return OOP_CONTINUE;
 }
@@ -358,15 +614,27 @@ static void udp_slot_start(int local_slot)
 
   if (session->must_send) {
     if (session->max_bit_rate > 0) {
-      session->tv_rate = OOP_TIME_NOW;
+      session->tv_send_rate = OOP_TIME_NOW;
       nepim_global.oop_src->on_time(nepim_global.oop_src,
-				    session->tv_rate,
+				    session->tv_send_rate,
 				    on_udp_rate_delay, slot);
     }
     else {
-      nepim_global.oop_src->on_fd(nepim_global.oop_src,
-				  sd, OOP_WRITE,
-				  on_udp_write, slot);
+      will_slot_write(slot);
+    }
+  }
+  else {
+    if (session->udp_keepalive_send) {
+
+      fprintf(stderr, 
+	      "DEBUG %s %s: %d %d-%d: scheduling sending of keepalives\n",
+	      __FILE__, __PRETTY_FUNCTION__,
+	      slot->udp_sd, slot->index, slot->index_remote);
+      
+      session->tv_keepalive = OOP_TIME_NOW;
+      nepim_global.oop_src->on_time(nepim_global.oop_src,
+				    session->tv_keepalive,
+				    on_udp_keepalive_time, slot);
     }
   }
 
@@ -383,13 +651,17 @@ static void udp_slot_start(int local_slot)
   nepim_sock_show_opt(stderr, sd);
 
   schedule_stat_interval(local_slot);
+
+  if (session->udp_keepalive_require)
+    nepim_schedule_keepalive_timer(slot, on_udp_keepalive_require);
 }
 
-static int parse_greetings(int sd, int *local_slot, int remote_slot,
+static int parse_greetings(nepim_greet_t *opt,
+			   int sd, int local_slot, int remote_slot,
 			   char *buf, const int buf_len,
-			   const struct sockaddr *remote, socklen_t remote_len)
+			   const struct sockaddr *remote,
+			   socklen_t remote_len)
 {
-  nepim_greet_t opt;
   int result;
   char *eos;
   char *past_end;
@@ -408,36 +680,36 @@ static int parse_greetings(int sd, int *local_slot, int remote_slot,
   fprintf(stderr, "DEBUG %s %s udp_buf_len=%d eos_offset=%d\n",
 	  __FILE__, __PRETTY_FUNCTION__, buf_len, past_end - buf);
 
-  result = nepim_parse_greetings(&opt, buf, past_end);
+  result = nepim_parse_greetings(opt, buf, past_end);
   if (result)
     return result;
 
-  assert(*local_slot == 0xFFFF);
-
-  *local_slot = nepim_slot_find_free(&slots);
-
   fprintf(stderr, 
-	  "%d %d-%d: UDP incoming: send=%d max_rate=%lld "
-	  "interval=%d duration=%d delay=%ld\n", 
-	  sd, *local_slot, remote_slot,
-	  opt.must_send, opt.bit_rate, 
-	  opt.stat_interval, opt.test_duration, 
-	  opt.write_delay);
-
-  nepim_slot_set_add(&slots, sd, 
-		     *local_slot, remote_slot,
-		     remote, remote_len, &opt);
+	  "%d %d-%d: UDP from (%s,%d): send=%d rate=%lld "
+	  "interval=%d duration=%d delay=%ld ka_send=%d ka_req=%d\n", 
+	  sd, 
+	  local_slot, remote_slot,
+	  inet_ntoa(((struct sockaddr_in *) remote)->sin_addr),
+	  nepim_sock_get_port((struct sockaddr *) remote),
+	  opt->must_send, opt->bit_rate, 
+	  opt->stat_interval, opt->test_duration, 
+	  opt->write_delay, opt->udp_keepalive_send,
+	  opt->udp_keepalive_require);
 
   return 0;
 }
 
 static void udp_consume(nepim_slot_t *slot, size_t len)
 {
-  fprintf(stderr, 
-	  "DEBUG FIXME %s %s: %d %d-%d: %d bytes\n",
-	  __FILE__, __PRETTY_FUNCTION__,
-	  slot->udp_sd, slot->index, slot->index_remote,
-	  len);
+  nepim_session_t *session = &slot->session;
+
+  /* any valid traffic accounted as keepalive */
+  ++slot->keepalives_recv;
+
+  session->byte_total_recv += len;
+  session->byte_interval_recv += len;
+  ++session->total_reads;
+  ++session->interval_reads;
 }
 
 static void *on_udp_segment(oop_source *src, int sd,
@@ -450,80 +722,95 @@ static void *on_udp_segment(oop_source *src, int sd,
     struct sockaddr_in6 inet6;
   } from;
   socklen_t fromlen = sizeof(from);
-  int version;
-  int local_slot;
-  int remote_slot;
-  uint32_t seq;
+  nepim_udp_hdr_t hdr;
 
   assert(sizeof(buf) == nepim_global.udp_read_size);
+  assert(event == OOP_READ);
+  assert(!unnused);
 
   rd = recvfrom(sd, buf, nepim_global.udp_read_size, 0, 
 		(struct sockaddr *) &from, &fromlen);
   if (rd < 1) {
     switch (errno) {
     case EINTR:
-      fprintf(stderr, "recvfrom: EINTR on UDP socket %d\n", sd);
-      return OOP_CONTINUE;
     case EAGAIN:
-      fprintf(stderr, "recvfrom: EAGAIN on UDP socket %d\n", sd);
+    case ECONNREFUSED:
+      nepim_usock_read_error(&udp_tab, sd, errno);
       return OOP_CONTINUE;
     }
 
-    fprintf(stderr, "recvfrom: failure on UDP socket %d\n", sd);
-
-    return OOP_CONTINUE;
-  }
-
-  if (rd < UDP_HEADER_LEN) {
     fprintf(stderr, 
-	    "recvfrom: too short UDP segment: %d bytes\n",
-	    rd);
+	    "%d: recvfrom: unexpected failure: errno=%d: %s\n",
+	    sd, errno, strerror(errno));
+
+    assert(0);
+
     return OOP_CONTINUE;
   }
 
-  assert(rd >= UDP_HEADER_LEN);
-  assert(rd < sizeof(buf));
+  assert(rd > 0);
+  assert(rd <= sizeof(buf));
 
-  version     = *buf;
-  local_slot  = ntohs((uint16_t) *(buf + 1));
-  remote_slot = ntohs((uint16_t) *(buf + 3));
-  seq         = ntohl((uint32_t) *(buf + 5));
+  nepim_usock_read_good(&udp_tab, sd);
 
-  fprintf(stderr, "DEBUG FIXME ERASEME %s %s forcing version=0 local_slot=0xFFFF\n",
-	  __FILE__, __PRETTY_FUNCTION__);
-  version    = 0;      /* DEBUG FIXME ERASEME */
-  local_slot = 0xFFFF; /* DEBUG FIXME ERASEME */
+  if (nepim_udp_hdr_parse(&hdr, buf, rd)) {
+    fprintf(stderr, 
+	    "%d: recvfrom: could not parse header from UDP segment: %d/%d bytes\n",
+	    sd, rd, nepim_global.udp_read_size);
+    return OOP_CONTINUE;
+  }
 
-  if (version) {
-    fprintf(stderr, "recvfrom: unknown UDP header version: %d\n",
-	    version);
+  if (hdr.version) {
+    fprintf(stderr, "%d: recvfrom: ignoring unknown UDP header version: %d\n",
+	    sd, hdr.version);
     return OOP_CONTINUE;
   }
 
   /* request for new slot? */
-  if (local_slot == 0xFFFF) {
+  if (hdr.type == UDP_TYPE_HELLO) {
     int result;
+    int local_slot;
+    nepim_greet_t opt;
 
     fprintf(stderr, 
-	    "DEBUG %s %s: new slot request: %d(%d-%d)\n",
+	    "DEBUG %s %s: new slot request: %d (%d-%d)\n",
 	    __FILE__, __PRETTY_FUNCTION__, 
-	    sd, local_slot, remote_slot);
+	    sd,
+	    nepim_uint16_read(buf + 1),
+	    nepim_uint16_read(buf + 3));
 
-    result = parse_greetings(sd, &local_slot, remote_slot, 
+    if (nepim_slot_find_addr(&slots, hdr.src_slot, 
+			     (const struct sockaddr *) &from,
+			     fromlen)) {
+      fprintf(stderr, 
+	      "%d %d-%d: normal behavior: ignoring existing remote src_slot/addr pair\n",
+	      sd, hdr.dst_slot, hdr.src_slot);
+      return OOP_CONTINUE;
+    }
+
+    result = parse_greetings(&opt, sd,
+			     hdr.dst_slot, hdr.src_slot, 
 			     buf + UDP_HEADER_LEN, 
 			     rd - UDP_HEADER_LEN,
 			     (const struct sockaddr *) &from,
 			     fromlen);
     if (result) {
       fprintf(stderr, 
-	      "%s: %s: bad greetings from UDP socket %d(%d-%d): %d\n",
-	      __FILE__, __PRETTY_FUNCTION__, sd, local_slot, 
-	      remote_slot, result);
+	      "%d %d-%d: recvfrom: bad greetings: %d\n",
+	      sd, hdr.dst_slot, hdr.src_slot, result);
       return OOP_CONTINUE;
     }
 
+    local_slot = nepim_slot_find_free(&slots);
+
     assert(local_slot < 0xFFFF);
 
+    nepim_slot_set_add(&slots, sd, 
+		       local_slot, hdr.src_slot,
+		       (const struct sockaddr *) &from, 
+		       fromlen,
+		       &opt);
+
     udp_slot_start(local_slot);
 
     return OOP_CONTINUE;
@@ -531,23 +818,36 @@ static void *on_udp_segment(oop_source *src, int sd,
   } /* new slot request */
 
   {
-    nepim_slot_t *slot = nepim_slot_set_search(&slots, local_slot);
+    nepim_slot_t *slot = nepim_slot_set_search(&slots, hdr.dst_slot);
 
     if (!slot) {
-      fprintf(stderr, "%d: unknown UDP local slot: %d\n",
-	      sd, local_slot);
+      fprintf(stderr,
+	      "%d: recvfrom: unknown local slot: %d\n",
+	      sd, hdr.dst_slot);
       return OOP_CONTINUE;
     }
 
-    if (slot->index_remote != remote_slot)
+    if (slot->index_remote != hdr.src_slot) {
       fprintf(stderr, 
-	      "%d %d: remote index mismatch: expected=%d got=%d\n",
-	      sd, local_slot, slot->index_remote, remote_slot);
+	      "%d %d: recvfrom: remote index mismatch: expected=%d got=%d\n",
+	      sd, hdr.dst_slot, slot->index_remote, hdr.src_slot);
+      return OOP_CONTINUE;
+    }
 
-    fprintf(stderr, 
-	    "DEBUG FIXME %s %s: existing slot: %d(%d-%d)\n",
-	    __FILE__, __PRETTY_FUNCTION__, 
-	    sd, local_slot, remote_slot);
+    if (slot->remote_len != fromlen) {
+      fprintf(stderr, 
+	      "%d %d-%d: recvfrom: bad address length: expected=%d got=%d\n",
+	      sd, hdr.dst_slot, hdr.src_slot, 
+	      slot->remote_len, fromlen);
+      return OOP_CONTINUE;
+    }
+
+    if (memcmp(&slot->remote, &from, fromlen)) {
+      fprintf(stderr, 
+	      "%d %d-%d: recvfrom: address mismatch\n",
+	      sd, hdr.dst_slot, hdr.src_slot);
+      return OOP_CONTINUE;
+    }
 
     udp_consume(slot, rd);
   }
@@ -555,7 +855,8 @@ static void *on_udp_segment(oop_source *src, int sd,
   return OOP_CONTINUE;
 }
 
-int nepim_udp_listener(const char *hostname, const char *portname, int mcast_join)
+int nepim_udp_listener(const char *hostname, const char *portname, 
+		       int mcast_join)
 {
   struct addrinfo hints;
   struct addrinfo *ai_res;
@@ -571,7 +872,7 @@ int nepim_udp_listener(const char *hostname, const char *portname, int mcast_joi
   hints.ai_addr = 0;
   hints.ai_canonname = 0;
 
-  result = getaddrinfo(hostname, 0, &hints, &ai_res);
+  result = getaddrinfo(hostname, portname, &hints, &ai_res);
   if (result) {
     fprintf(stderr, "getaddrinfo(%s, %s): %s\n",
             hostname, portname, gai_strerror(result));
@@ -583,8 +884,8 @@ int nepim_udp_listener(const char *hostname, const char *portname, int mcast_joi
 
 #if 0
     fprintf(stderr, 
-	    "UDP listener: host=%s,%s len=%d family=%d type=%d proto=%d\n",
-	    hostname, portname, 
+	    "UDP listener: host=%s,%s(%d) len=%d family=%d type=%d proto=%d\n",
+	    hostname, portname, nepim_sock_get_port(ai->ai_addr),
 	    ai->ai_addrlen, ai->ai_family, 
 	    ai->ai_socktype, ai->ai_protocol);
 #endif
@@ -600,8 +901,8 @@ int nepim_udp_listener(const char *hostname, const char *portname, int mcast_joi
 			     mcast_join);
     if (sd < 0) {
       fprintf(stderr, 
-	      "spawn_udp_listener: UDP listener socket failed for %s,%s: %d\n",
-	      hostname, portname, sd);
+	      "spawn_udp_listener: UDP listener socket failed for %s,%s(%d): %d\n",
+	      hostname, portname, nepim_sock_get_port(ai->ai_addr), sd);
       break;
     }
 
@@ -609,11 +910,13 @@ int nepim_udp_listener(const char *hostname, const char *portname, int mcast_joi
 			  	sd, OOP_READ,
 				on_udp_segment, 0);
 
+    nepim_usock_set_add(&udp_tab, sd);
+
     ++udp_listeners;
 
     fprintf(stderr, 
-	    "%d: UDP socket listening on %s,%s\n",
-	    sd, hostname, portname);
+	    "%d: UDP socket listening on %s,%s(%d)\n",
+	    sd, hostname, portname, nepim_sock_get_port(ai->ai_addr));
 
     nepim_sock_show_opt(stderr, sd);
   }
